require 'selenium-webdriver'
require 'yaml'
require 'page-object/platforms/selenium_webdriver/element'
require 'page-object'
require 'active_support'
require 'page/driver'
require 'page/gems_overriding'

class Page
  include PageObject
  include GemsOverriding
  common_config = YAML.load_file('lib/configs/main_config.yml')
  STAND = common_config['stand'].freeze
  CONFIG = common_config.merge!(YAML.load_file("lib/configs/#{STAND}.yml")).freeze

  def self.driver
    Driver.instance.browser
  end

  def initialize(visit = false)
    super(Page.driver, visit)
  end

  # direct upload files via window upload dialog
  def upload_file(element, file_path)
    element.send_keys(file_path.tr('/', '\\'))
  end

  # deal with code mirror text areas
  def type_code_mirror(id, text)
    execute_script("$('##{id}').val('#{text}');")
    execute_script("$('##{id}').keyup();")
  end

  def code_mirror(id)
    execute_script("return $('##{id}').val();")
  end

  # deal with ck editor text areas
  def type_ck_editor(id, text)
    execute_script("CKEDITOR.instances['#{id}'].insertText('#{text}');")
  end

  # close all windows except current
  def close_other_windows
    cur_window = browser.window_handle
    browser.window_handles.each do |window|
      next if window.eql?(cur_window)

      browser.switch_to.window(window)
      browser.close
    end
    browser.switch_to.window(cur_window)
  end

  # switch to next window and close current
  def switch_next_window
    cur_window = browser.window_handle
    browser.close
    browser.window_handles.each do |window|
      next if window.eql?(cur_window)

      browser.switch_to.window(window)
      break
    end
  end
end
